from itertools import chain
from typing import Union

import pygame

from rtsgame.src.Client.HPBar import HPBar
from rtsgame.src.Client.TileSprite import TileSprite
from rtsgame.src.Client.UI.Widget import Widget
from rtsgame.src.utility.constants import PLAYER_HEALTH
from rtsgame.src.utility.utilities import Vector


class Camera(Widget):
    def __init__(self, world_camera_position: Vector, size):
        super().__init__()
        self._box = pygame.Rect(world_camera_position, size)
        self._sprites = []
        self._tiles = []
        self.hpbar = HPBar(PLAYER_HEALTH, PLAYER_HEALTH)
        self.add_child(self.hpbar, Vector(0, -100))

    @property
    def size(self):
        return Vector(*self._box.size)

    def draw(self, surface: pygame.Surface,
             abs_position: Union[Vector, tuple]):
        world_pos = Vector(*self._box.topleft)

        for sprite in chain(self._tiles, self._sprites):
            if not self.is_visible(sprite):
                continue

            rel_pos = sprite.position - world_pos
            abs_child_pos = abs_position + rel_pos
            sprite.draw(surface, abs_child_pos)

        super().draw(surface, abs_position)

    def set_sprites(self, sprites):
        self._sprites = sprites

    def set_center(self, world_position: Vector):
        self._box.center = world_position

    def set_map(self, tile_map):
        self._tiles = []
        for i, row in enumerate(tile_map):
            for j, tile_type in enumerate(row):
                self._tiles.append(TileSprite(tile_type, j, i))

    def set_hp(self, cur_hp):
        self.hpbar.hp = cur_hp

    def is_visible(self, sprite):
        if sprite.box is None:
            return True

        return self._box.colliderect(sprite.box)



import json
from random import randint

from pygame import Rect

from .Entity import MeleeEnemy
from ..utility.constants import *


class EnemyFactory:
    def __init__(self):
        pass

    def generate_enemy(self):
        raise NotImplementedError


class MeleeEnemyFactory(EnemyFactory):
    def __init__(self):
        super().__init__()
        self.box_width = ENEMY_BOX_SIZE
        self.box_height = ENEMY_BOX_SIZE
        self.min_health = 10
        self.max_health = self.min_health * 2
        self.damage = 2
        self.id = 2

    def generate_enemy(self):
        current_id = self.id
        self.id += 1
        box = generate_box(self.box_width, self.box_height)
        damage = self.damage
        health = generate_enemy_health(self.min_health, self.max_health)
        direction = generate_random_direction()
        print("BOX {} health {} direction {} id {}".format(box, health, direction, current_id))
        return MeleeEnemy().set_id(current_id).set_box(box).set_damage(damage) \
            .set_health(health).set_direction(direction).set_attack_reload(
            ATTACK_RELOAD).set_velocity(ENEMY_VELOCITY)

    def load_difficulty_from_file(self, name):

        obj = json.load(open("src/utility/difficulties/{}.json".format(name)))
        self.box_width = obj["enemies"][0]["box_width"]
        self.box_height = obj["enemies"][0]["box_height"]
        self.min_health = obj["enemies"][0]["min_health"]
        self.max_health = obj["enemies"][0]["max_health"]


def generate_random_direction():
    return 1, 0


def generate_enemy_damage():
    return 10


def generate_enemy_health(min_health, max_health):
    return randint(min_health, max_health + 1)


def generate_box(width, height):
    return Rect(0, 0, width, height)



MAX_X = 500
MAX_Y = 500
VISION_RANGE = 500
ATTACK_RANGE = 100
ATTACK_RELOAD = 0.5
PROJECTILE_DAMAGE = 25
PROJECTILE_VELOCITY = 5
ENEMY_VELOCITY = 4
WALL = -1
FLOOR = 0
STONE = 1
FREE_SPACE = 0
MAP_SCALE = 100
PLAYER_START_DAMAGE = 100
PLAYER_HEALTH = 100
PLAYER_VELOCITY = 15
PORT = 8080
WIDTH = 100
HEIGHT = 100
MIN_ROOM_LEN = 10
MAX_ROOM_LEN = 10
ROOMS = 2
RANDOM_CONNECTIONS = 0
ENEMY_BOX_SIZE = 50
# TODO: нормальный box size
PLAYER_HEIGHT = MAP_SCALE // 2
PLAYER_WIDTH = MAP_SCALE // 2
PLAYER_RELOAD = 0.4

MOVE_UPDATE = 100
PIXEL_SCALE = 100



import logging

import pygame

from rtsgame.src.Client.EntitySprite import EntitySpriteManager


class Game:
    def __init__(self):
        logging.basicConfig(level=logging.INFO)
        self.screen = None
        self.clock = None
        self.active_window = None
        self.running = True

    def start_main_window(self, level_map):
        from rtsgame.src.Client.MainWindow import MainWindow
        main_window = MainWindow((1000, 1000), self.active_window.sio)
        main_window.main_camera.set_map(level_map)
        self.active_window = main_window

    def run(self, ip):
        pygame.init()
        self.screen = pygame.display.set_mode((1000, 1000))

        EntitySpriteManager.load_entity_config('src/utility/animations/'
                                               'melee_animations.json')
        EntitySpriteManager.load_entity_config('src/utility/animations/'
                                               'player1_animations.json')
        EntitySpriteManager.load_entity_config('src/utility/animations/'
                                               'player2_animations.json')
        self.clock = pygame.time.Clock()

        from rtsgame.src.Client.TextWindow import WaitWindow
        self.active_window = WaitWindow()

        from rtsgame.src.Client.SIOServer import sio, run
        self.active_window.set_sio(sio)
        run(ip)

        while self.running:
            self.clock.tick(30)
            self.screen.fill((0, 0, 0))
            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    self.running = False
                else:
                    self.active_window.accept_event(event)

            self.active_window.draw(self.screen)
            pygame.display.update()
            pygame.event.pump()

        sio.disconnect()

    def accept_action(self, action):
        self.active_window.accept_action(action)


game = Game()



import json
import os

import pygame

from rtsgame.src.Client.Animation import parse_descriptions
from rtsgame.src.Client.Sprite import Sprite
from rtsgame.src.utility.constants import PIXEL_SCALE, MAP_SCALE
from rtsgame.src.utility.utilities import Vector


class EntitySpriteManager:
    _animations = {}

    def __new__(cls, *args, **kwargs):
        raise NotImplementedError("Sprite Manager is static class")

    @classmethod
    def register_animations(cls, entity_type, animations):
        if cls.has_type(entity_type):
            raise RuntimeError("Entity_type {}"
                               "is already registered")

        cls._animations[entity_type] = animations

    @classmethod
    def get_sprite(cls, entity_type, animation_name, frame):
        if not cls.has_animation(entity_type, animation_name):
            raise RuntimeError("Entity {} doesn't exist or doesn't have "
                               "animation {}"
                               .format(entity_type, animation_name))
        return cls._animations[entity_type][animation_name][frame]

    @classmethod
    def get_offset(cls, entity_type, animation_name):
        if not cls.has_animation(entity_type, animation_name):
            raise RuntimeError("Entity {} doesn't exist or doesn't have "
                               "animation {}"
                               .format(entity_type, animation_name))
        return cls._animations[entity_type][animation_name].offset

    @classmethod
    def has_type(cls, entity_type):
        return entity_type in cls._animations

    @classmethod
    def has_animation(cls, entity_type, animation_name):
        return (cls.has_type(entity_type) and
                animation_name in cls._animations[entity_type])

    @classmethod
    def load_entity_config(cls, filename):

        with open(os.path.join(os.path.dirname(os.path.abspath(__file__)),
                               "../../" + filename)) as file:
            config = json.load(file)

        all_animations = parse_descriptions(config['move'])
        all_animations.update(parse_descriptions(config['attack']))
        cls.register_animations(config['entity_type'], all_animations)


class EntitySprite(Sprite):
    def __init__(self, info, debug=False):
        super().__init__()
        self.debug = debug
        self.info = info
        self.x = info[0][0] / MAP_SCALE * PIXEL_SCALE
        self.y = info[0][1] / MAP_SCALE * PIXEL_SCALE
        self.box = None

    @property
    def type(self):
        return self.info[1]

    @property
    def animation_name(self):
        return self.info[2]

    @property
    def frame(self):
        return self.info[3]

    @property
    def position(self):
        return Vector(self.x, self.y)

    @property
    def sprite(self):
        return EntitySpriteManager.get_sprite(self.type, self.animation_name,
                                              self.frame)

    @property
    def health(self):
        return self.info[4][0]

    @property
    def offset(self):
        return EntitySpriteManager.get_offset(self.type, self.animation_name)

    def draw(self, surface: pygame.Surface, abs_position: Vector):
        if self.debug:
            pygame.draw.rect(surface, (255, 255, 255),
                             (
                                 abs_position, (PIXEL_SCALE / 2, PIXEL_SCALE / 2)))
        surface.blit(self.sprite, abs_position - self.offset)

        super().draw(surface, abs_position)



import os

from .constants import MAP_SCALE


def cls_init(cls):
    cls.cls_init()
    return cls


def join_paths(folder, filenames):
    return [os.path.join(folder, f) for f in filenames]


class Vector(tuple):
    def __new__(cls, x, y):
        return super().__new__(cls, (x, y))

    def __init__(self, x, y):
        super().__init__()

    def __add__(self, other):
        return Vector(self[0] + other[0],
                      self[1] + other[1])

    def __sub__(self, other):
        return Vector(self[0] - other[0],
                      self[1] - other[1])

    def __truediv__(self, other):
        return Vector(self[0] / 2, self[1] / 2)

    @property
    def x(self):
        return self[0]

    @property
    def y(self):
        return self[1]


def load_sprites(files):
    import pygame
    return [pygame.image.load(file) for file in files]


def level_to_tile(level):
    _tile = [[] for _ in range(len(level) * MAP_SCALE)]
    for i in range(len(level)):
        for j in range(len(level[i])):
            text = level[i][j]
            for k in range(MAP_SCALE):
                _tile[i * MAP_SCALE + k] += [text] * MAP_SCALE
    return _tile



import os
from MainMenu import run_server
import multiprocessing
from queue import Queue
import time
os.chdir("..")

def f(test):
    st = time.time()
    test()
    print(time.time() - st)
queue = Queue()

def test_map_receive():
    server_thread = multiprocessing.Process(target=run_server,
                                            args=["Singleplayer"])
    server_thread.start()
    time.sleep(3)
    from src.Client.SIOServer import sio, run
    sio.on("message")(on_message)
    sio.on("disconnect")(on_disconnect)
    run("localhost")
    assert queue.get()[0] == "MAP"
    server_thread.terminate()

def on_message(data):
    queue.put(data)

def on_disconnect():
    print("disco")
    exit(0)
    



from abc import ABCMeta, abstractmethod

import pygame

from rtsgame.src.Client.UI.Widget import Widget
from rtsgame.src.utility.utilities import Vector


class Sprite(Widget, metaclass=ABCMeta):
    def __init__(self):
        super().__init__()
        self.box = None

    @abstractmethod
    def position(self):
        pass

    @abstractmethod
    def sprite(self):
        pass

    @abstractmethod
    def draw(self, surface: pygame.Surface, abs_position: Vector):
        super().draw(surface, abs_position)



import multiprocessing
import time
import tkinter as tk
from tkinter import font as tkfont
from tkinter import messagebox


# import eventlet
# eventlet.monkey_patch()


class Master(tk.Tk):

    def __init__(self, *args, **kwargs):
        tk.Tk.__init__(self, *args, **kwargs)
        self.geometry("400x200")
        self.resizable(width=False, height=False)
        self.title_font = tkfont.Font(family='Helvetica', size=18,
                                      weight="bold", slant="italic")

        container = tk.Frame(self)
        container.pack(side="top", fill="both", expand=True)
        container.grid_rowconfigure(0, weight=1)
        container.grid_columnconfigure(0, weight=1)

        self.frames = {}
        for Frame in (MainMenu, MultiPlayer, Loading, Settings, Connect):
            page_name = Frame.__name__
            frame = Frame(parent=container, controller=self)
            self.frames[page_name] = frame

            frame.grid(row=0, column=0, sticky="nsew")

        self.show_frame("MainMenu")

    def exit_app(self):
        input_ = tk.messagebox.askquestion('Exit Application',
                                           'Are you sure you want to exit the '
                                           'application',
                                           icon='warning')
        if input_ == 'yes':
            self.destroy()

    def show_frame(self, page_name):
        frame = self.frames[page_name]
        frame.tkraise()

    def single_player_start(self):
        from rtsgame.src.Client.Game import game
        server_thread = multiprocessing.Process(target=run_server,
                                                args=["Singleplayer"])
        server_thread.start()
        self.destroy()

        time.sleep(5)
        game.run("localhost")

    def connect(self, ip: tk.StringVar):
        from rtsgame.src.Client.Game import game
        self.destroy()
        game.run(ip.get())

    def create_multiplayer_server(self):
        from rtsgame.src.Client.Game import game
        server_thread = multiprocessing.Process(target=run_server,
                                                args=["Multiplayer"])
        server_thread.start()
        self.destroy()

        time.sleep(5)

        game.run("localhost")


class MainMenu(tk.Frame):

    def __init__(self, parent, controller):
        tk.Frame.__init__(self, parent)

        label = tk.Label(self, text="Main menu", font=controller.title_font)
        label.pack(side="top", fill="x", pady=10)

        singleplayer_button = tk.Button(self, text="Single Player",
                                        command=lambda: [
                                            controller.show_frame("Loading"),
                                            controller.single_player_start()])
        multiplayer_button = tk.Button(self, text="Multi Player",
                                       command=lambda: controller.show_frame(
                                           "MultiPlayer"))
        settings_button = tk.Button(self, text="Settings",
                                    command=lambda: controller.show_frame(
                                        "Settings"))
        exit_button = tk.Button(self, text="Exit",
                                command=controller.exit_app)
        singleplayer_button.pack()
        multiplayer_button.pack()
        settings_button.pack()
        exit_button.pack()


class MultiPlayer(tk.Frame):

    def __init__(self, parent, controller):
        tk.Frame.__init__(self, parent)

        host_game_button = tk.Button(self, text="Host Game",
                                     command=lambda: controller.create_multiplayer_server())
        connect_button = tk.Button(self, text="Connect",
                                   command=lambda: controller.show_frame(
                                       "Connect"))
        back_button = tk.Button(self, text="Back",
                                command=lambda: controller.show_frame(
                                    "MainMenu"))
        host_game_button.pack()
        connect_button.pack()
        back_button.pack()


class Loading(tk.Frame):

    def __init__(self, parent, controller):
        tk.Frame.__init__(self, parent)

        label = tk.Label(self, text="Loading ...", font=controller.title_font)
        label.pack(side="top", fill="x", pady=10)

        cancel_button = tk.Button(self, text="Cancel",
                                  command=lambda: controller.show_frame(
                                      "MainMenu"))
        cancel_button.pack()


class Settings(tk.Frame):

    def __init__(self, parent, controller):
        tk.Frame.__init__(self, parent)

        label = tk.Label(self, text="Settings", font=controller.title_font)
        label.pack(side="top", fill="x", pady=10)

        back_button = tk.Button(self, text="Back",
                                command=lambda: controller.show_frame(
                                    "MainMenu"))
        back_button.pack()


class Connect(tk.Frame):
    def __init__(self, parent, controller):
        ip = tk.StringVar()
        tk.Frame.__init__(self, parent)

        label = tk.Label(self, text="Connect", font=controller.title_font)
        label.pack(side="top", fill="x", pady=10)

        ip_entry = tk.Entry(self, textvariable=ip)
        ip_entry.pack()

        connect_button = tk.Button(self, text="Connect",
                                   command=lambda: [
                                       controller.show_frame("Loading"),
                                       controller.connect(ip)])
        back_button = tk.Button(self, text="Back",
                                command=lambda: controller.show_frame(
                                    "MainMenu"))
        connect_button.pack()
        back_button.pack()


def run_server(mode):
    import rtsgame.App
    rtsgame.App.start_game(mode)

def main():
    app = Master()
    app.protocol("WM_DELETE_WINDOW", app.exit_app)
    app.mainloop()

if __name__ == "__main__":
    main()





import logging

from rtsgame.src.Client.TextWidget import TextWidget
from rtsgame.src.Client.UI.Window import Window
from rtsgame.src.utility.utilities import Vector


class WaitWindow(Window):
    def __init__(self):
        super().__init__()
        text = TextWidget('Waiting! Be comfy, take a cookie.')
        self.add_child(text, Vector(500, 400))
        self.map = None

    def accept_action(self, action):
        if not isinstance(action, list):
            raise RuntimeError(
                "Action is not of type list. Don't know what to do with it")

        if action[0] == "MAP":
            self.map = action[1]
            logging.info("Acquired map")
            self.sio.emit("message", "MAP_RECEIVED")

        if action[0] == "START_GAME":
            from rtsgame.src.Client.Game import game
            game.start_main_window(self.map)



import os

from src.Server.Entity import PlayerEntity
from src.Server.Logic import Logic
from src.Server.WorldState import world
from src.utility.constants import *

os.chdir("..")

logic = Logic()


def test_player_spawn():
    logic.spawn_system.create_player(0)
    assert isinstance(world.entity[0], PlayerEntity)
    assert len(world.entity) == 1
    assert len(world.enemies) == 0
    assert len(world.movable_entities) == 1


def test_player_spawn_propereties():
    assert world.get_health(0) == PLAYER_HEALTH
    assert world.get_damage(0) == PLAYER_START_DAMAGE
    assert world.get_velocity(0) == PLAYER_VELOCITY


def test_single_enemy_spawn():
    logic.spawn_system.create_enemy()
    assert len(world.entity) == 2
    assert len(world.enemies) == 1


def test_damage_deal():
    logic.damage_system.deal_damage(0, 2)
    enemy_start_health = world.get_health(2)
    player_damage = world.get_damage(0)
    if player_damage >= enemy_start_health:
        assert len(world.dead_entities) == 1
    else:
        assert world.get_health(2) == enemy_start_health - player_damage


def test_player_move():
    initial_position = world.get_box(0).center
    for direction in [(0, 1), (1, 0), (-1, 0), (0, -1)]:
        world.set_direction(0, direction)
        dx, dy = [i * world.get_velocity(0) for i in direction]

        temp_box = world.get_box(0).move(dx, dy)
        if logic.geometry_system.collide_with_wall(temp_box):
            logic.move(0)
            assert initial_position == world.get_box(0).center

        for other_entity_id in world.entity.keys():
            if logic.geometry_system.collide(temp_box, world.get_box(other_entity_id)) and 0 != other_entity_id:
                logic.move(0)
                assert initial_position == world.get_box(0).center
                break
        else:
            logic.move(0)
            new_position = world.get_box(0).center
            assert new_position != initial_position
            initial_position = new_position



class ActionBuilder:
    def __init__(self):
        self._action = {'position': None, 'type': None, 'animation_name': None, 'frame': None,
                        'hp': None}

    def set_position(self, left_top_corner):
        self._action['position'] = left_top_corner
        return self

    def set_type(self, type_):
        self._action['type'] = type_
        return self

    def set_animation_state(self, name, frame):
        self._action['animation_name'] = name
        self._action['frame'] = frame
        return self

    def get_action(self):
        return tuple(self._action.values())

    def set_hp(self, curr_hp, max_hp):
        self._action['hp'] = (curr_hp, max_hp)
        return self



from copy import deepcopy

from rtsgame.src.Server import Map
from rtsgame.src.utility.constants import *


class WorldState:
    def __init__(self, game_mode='Singleplayer'):
        self.game_mode = game_mode
        self.entity = {}
        self.movable_entities = set()
        self.enemies = set()
        self.projectiles = set()
        self.dead_entities = []
        self.map = Map.Map(width=WIDTH, height=HEIGHT, max_rooms=ROOMS,
                           min_room_len=MIN_ROOM_LEN,
                           max_room_len=MAX_ROOM_LEN,
                           random_connections=RANDOM_CONNECTIONS)
        self.first_player_glare = deepcopy(self.map.level)
        self.second_player_glare = deepcopy(self.map.level)
        self.first_player_id = 0
        self.second_player_id = 1

    def get_game_mode(self):
        return self.game_mode

    def set_game_mode(self, game_mode):
        self.game_mode = game_mode
        if game_mode == 'Singleplayer':
            self.player_dead = [False]
        else:
            self.player_dead = [False, False]

    def get_first_player_id(self):
        return self.first_player_id

    def get_second_player_id(self):
        return self.second_player_id

    def get_position(self, entity_id):
        return (
            self.get_box(entity_id).centerx, self.get_box(entity_id).centery)

    def get_direction(self, entity_id):
        return self.entity[entity_id].get_direction()

    def get_velocity(self, entity_id):
        return self.entity[entity_id].get_velocity()

    def get_damage(self, entity_id):
        return self.entity[entity_id].get_damage()

    def get_health(self, entity_id):
        return self.entity[entity_id].get_health()

    def get_box(self, entity_id):
        return self.entity[entity_id].get_box()

    def get_last_attack(self, entity_id):
        return self.entity[entity_id].get_last_attack()

    def get_attack_reload(self, entity_id):
        return self.entity[entity_id].get_attack_reload()

    def set_last_attack(self, entity_id, last_attack):
        self.entity[entity_id].set_last_attack(last_attack)

    def set_attack_reload(self, entity_id, attack_reload):
        self.entity[entity_id].set_attack_reload(attack_reload)

    def set_box(self, entity_id, box):
        self.entity[entity_id].set_box(box)

    def set_position(self, entity_id, position):
        self.entity[entity_id].set_position(position)

    def set_direction(self, entity_id, direction):
        self.entity[entity_id].set_direction(direction)

    def set_velocity(self, entity_id, velocity):
        self.entity[entity_id].set_velocity(velocity)

    def set_damage(self, entity_id, damage):
        self.entity[entity_id].set_damage(damage)

    def set_health(self, entity_id, health):
        self.entity[entity_id].set_health(health)

    def delete_entity(self, entity_id):
        self.movable_entities.discard(entity_id)
        del self.entity[entity_id]
        self.enemies.discard(entity_id)
        self.projectiles.discard(entity_id)


world = WorldState()

def recreate_world():
    global world
    world = WorldState()


from threading import Thread

import socketio

from rtsgame.src.Client.Game import game
from rtsgame.src.utility.constants import PORT

sio = socketio.Client()
sio_thread = None

@sio.on('message')
def accept_action(data):
    game.accept_action(data)


@sio.on('connect')
def on_connect():
    sio.emit('message', 'PLAYER_CONNECTED')


@sio.on('disconnect')
def on_disconnect():
    game.running = False
    exit(0)


def run(ip):
    global sio_thread
    sio.connect(f"http://{ip}:{PORT}")

    sio_thread = Thread(target=sio.wait)
    sio_thread.setDaemon(True)
    sio_thread.start()



from math import sin, cos, inf

from rtsgame.src.Server.Entity import PlayerEntity, Enemy
from rtsgame.src.Server.WorldState import world
from rtsgame.src.utility.constants import *


class GeometrySystem:

    def get_visible_tiles(self, entity_id):
        # Обновлять glare для правильного игрока
        glare_map = None
        visible_tiles = []
        if entity_id == world.first_player_id:
            glare_map = world.first_player_glare
        if entity_id == world.second_player_id:
            glare_map = world.second_player_id

        for i in range(360):
            deg = i * 3.1415 / 180
            x0 = world.get_box(entity_id).centerx
            y0 = world.get_box(entity_id).centery
            x = round(cos(deg) * VISION_RANGE) + world.get_box(
                entity_id).centerx
            y = round(sin(deg) * VISION_RANGE) + world.get_box(
                entity_id).centery

            diag_dist = max(abs(x - x0), abs(y - y0))

            for j in range(diag_dist):
                tx = round(x0 + (j / diag_dist) * (x - x0))
                ty = round(y0 + (j / diag_dist) * (y - y0))

                if (tx < 0 or tx >= world.map.width * MAP_SCALE) or (
                        ty < 0 or ty >= world.map.height * MAP_SCALE):
                    break
                if world.map.level[tx][ty] == WALL:
                    visible_tiles.append((tx, ty))
                    break
                visible_tiles.append((tx, ty))
                if glare_map is not None:
                    glare_map[tx][ty] = 1
        return visible_tiles

    @staticmethod
    def get_squared_distance(entity_id1, entity_id2):
        return (world.get_box(entity_id1).centerx - world.get_box(entity_id2).centerx) ** 2 + \
               (world.get_box(entity_id1).centery - world.get_box(entity_id2).centery) ** 2

    # TODO: нормальная система зрения
    @staticmethod
    def _is_visible(position1: tuple, position2: tuple) -> bool:
        return ((position1[0] - position2[0]) ** 2 + (
                position1[1] - position2[1]) ** 2) ** 0.5 < VISION_RANGE

    @staticmethod
    def _is_attackable(position1: tuple, position2: tuple) -> bool:
        return ((position1[0] - position2[0]) ** 2 + (
                position1[1] - position2[1]) ** 2) ** 0.5 < ATTACK_RANGE

    def get_visible_entities(self, entity_id) -> list:
        entities_id = []
        for other_entity_id in world.entity.keys():
            if self._is_visible(world.get_position(entity_id),
                                world.get_position(other_entity_id)):
                entities_id.append(other_entity_id)
        return entities_id

    def get_attackable_entites(self, entity_id) -> list:
        entities_id = []
        if isinstance(world.entity[entity_id], PlayerEntity):
            EnemyClass = Enemy
        if isinstance(world.entity[entity_id], Enemy):
            EnemyClass = PlayerEntity

        for other_entity_id in world.entity.keys():
            if isinstance(world.entity[other_entity_id], EnemyClass):
                if self._is_attackable(world.get_position(entity_id), world.get_position(other_entity_id)) \
                        and other_entity_id != entity_id:
                    entities_id.append(other_entity_id)
        return entities_id

    def generate_npc_movement(self, npc_id):
        if world.entity[npc_id].get_aim() is None:
            return (0, 0)
        target = world.entity[npc_id].get_aim()
        current = world.get_position(npc_id)
        directions = [(0, 0), (0, 1), (1, 1), (1, 0), (-1, 0), (-1, -1), (0, -1), (1, -1), (-1, 1)]
        ans = (0, 0)
        min_dist = 10 ** 10
        for direction in directions:
            new_cur = (current[0] + direction[0], current[1] + direction[1])
            if ((new_cur[0] - target[0]) ** 2 + (new_cur[1] - target[1]) ** 2) < min_dist:
                min_dist = (new_cur[0] - target[0]) ** 2 + (new_cur[1] - target[1]) ** 2
                ans = direction
        return ans

    # Провекра пересечения хитбоксов
    @staticmethod
    def collide(box1, box2):
        return box1.colliderect(box2)

    @staticmethod
    def collide_with_wall(box):  # Саша проверь
        non_passable_textures = {WALL, STONE}
        return world.map.get(
            *box.topleft) in non_passable_textures or world.map.get(
            *box.topright) \
               in non_passable_textures or world.map.get(
            *box.bottomleft) in non_passable_textures or \
               world.map.get(*box.bottomright) in non_passable_textures

    def find_aim(self, entity_id):
        if world.game_mode == "Multiplayer":
            if not world.player_dead[world.get_first_player_id()]:
                dist_to_first_player = self.get_squared_distance(entity_id, world.get_first_player_id())
            else:
                dist_to_first_player = inf

            if not world.player_dead[world.get_second_player_id()]:
                dist_to_second_player = self.get_squared_distance(entity_id, world.get_second_player_id())
            else:
                dist_to_second_player = inf

            dist_to_aim = min(dist_to_first_player, dist_to_second_player)

            if dist_to_first_player < dist_to_second_player:
                probable_aim = world.get_box(world.get_first_player_id()).center

            else:
                probable_aim = world.get_box(world.get_second_player_id()).center

            if dist_to_aim < VISION_RANGE ** 2:
                world.entity[entity_id].set_aim(probable_aim)
        if world.game_mode == "Singleplayer":
            dist_to_first_player = self.get_squared_distance(entity_id, world.get_first_player_id())
            if dist_to_first_player < VISION_RANGE ** 2:
                world.entity[entity_id].set_aim(world.get_box(world.get_first_player_id()).center)



import logging

import pygame

from rtsgame.src.utility.utilities import Vector


class Widget:
    def __init__(self):
        self.children = {}

    def add_child(self, child, position: Vector):
        id_ = id(child)

        if id_ in self.children:
            raise RuntimeError("Child {} is already connected as child"
                               .format(id_))
        self.children[id_] = (child, position)
        logging.info("Added child")

    def remove_child(self, child):
        id_ = id(child)
        del self.children[id_]

    def draw(self, surface: pygame.Surface, abs_position: Vector):
        # TODO blit background
        for child, rel_pos in self.children.values():
            child.draw(surface, abs_position + rel_pos)



import pygame

from rtsgame.src.Client.Sprite import Sprite
from rtsgame.src.utility.constants import WALL, STONE, FLOOR, PIXEL_SCALE
from rtsgame.src.utility.utilities import Vector

_surfaces = {
    WALL: {
        0: pygame.Surface((PIXEL_SCALE, PIXEL_SCALE)),
        1: pygame.Surface((PIXEL_SCALE, PIXEL_SCALE))
    },
    FLOOR: {
        0: pygame.Surface((PIXEL_SCALE, PIXEL_SCALE)),
        1: pygame.Surface((PIXEL_SCALE, PIXEL_SCALE))
    },
    STONE: {
        0: pygame.Surface((PIXEL_SCALE, PIXEL_SCALE)),
        1: pygame.Surface((PIXEL_SCALE, PIXEL_SCALE))
    }
}

_surfaces[WALL][0].fill((100, 100, 100))
_surfaces[WALL][1].fill((100, 100, 100))
_surfaces[FLOOR][0].fill((100, 0, 0))
_surfaces[FLOOR][1].fill((100, 0, 0))
_surfaces[STONE][0].fill((100, 100, 0))
_surfaces[STONE][1].fill((100, 100, 0))

pygame.draw.rect(_surfaces[WALL][1], (255, 0, 0),
                 (0, 0, PIXEL_SCALE, PIXEL_SCALE))
pygame.draw.rect(_surfaces[FLOOR][1], (255, 0, 0),
                 (0, 0, PIXEL_SCALE, PIXEL_SCALE))
pygame.draw.rect(_surfaces[STONE][1], (255, 0, 0),
                 (0, 0, PIXEL_SCALE, PIXEL_SCALE))

_surfaces[WALL][0] = _surfaces[WALL][0].convert_alpha()
_surfaces[WALL][1] = _surfaces[WALL][1].convert_alpha()
_surfaces[FLOOR][0] = _surfaces[FLOOR][0].convert_alpha()
_surfaces[FLOOR][1] = _surfaces[FLOOR][1].convert_alpha()
_surfaces[STONE][0] = _surfaces[STONE][0].convert_alpha()
_surfaces[STONE][1] = _surfaces[STONE][1].convert_alpha()


class TileSprite(Sprite):
    def __init__(self, tile_type, col, row, tile_state=0):
        super().__init__()
        self.tile_type = tile_type
        self.tile_state = tile_state
        self.col = col
        self.row = row
        self.box = pygame.Rect(self.row * PIXEL_SCALE, self.col * PIXEL_SCALE,
                               PIXEL_SCALE, PIXEL_SCALE)

    def set_visible(self):
        self.tile_state = 1

    @property
    def x(self):
        return self.box.left

    @property
    def y(self):
        return self.box.top

    @property
    def position(self):
        return Vector(*self.box.topleft)

    @property
    def sprite(self):
        return _surfaces[self.tile_type][self.tile_state]

    def draw(self, surface: pygame.Surface, abs_position: Vector):
        surface.blit(self.sprite, abs_position)

        super().draw(surface, abs_position)



import logging
import queue
from threading import Thread

import eventlet
import socketio

from rtsgame.src.utility.constants import PORT

player_disconnected = False


class Server:
    def __init__(self, port=PORT):
        super().__init__()
        self.action_queue = queue.Queue()
        self.connections = {}
        self.rconnections = {}
        self.activeConnections = 0
        self.sio = socketio.Server(async_mode='eventlet')
        self.app = socketio.WSGIApp(self.sio)
        self.port = port

        self.sio.on('message')(self.on_message)
        self.sio.on('connect')(self.on_connect)
        self.sio.on('disconnect')(self.on_disconnect)

    def on_message(self, sid, *data):
        # todo : ПОДУМАТЬ
        if len(data) == 1:
            self.action_queue.put((self.rconnections[sid], data[0]))
        else:
            self.action_queue.put((self.rconnections[sid], data))

    def on_connect(self, sid, environ):
        self.connections[self.activeConnections] = sid
        self.rconnections[sid] = self.activeConnections
        self.activeConnections += 1

    def on_disconnect(self, sid):
        global player_disconnected
        player_disconnected = True

    def send_obj_all_players(self, obj):
        for player_id in range(len(self.connections)):
            self.send_obj_to_player(obj, player_id)

    def start(self):
        thread = Thread(target=self.run)
        thread.setDaemon(True)
        thread.start()

    def run(self):
        try:
            eventlet.wsgi.server(eventlet.listen(('', self.port)), self.app)
        except Exception as e:
            logging.error("Error in server {}".format(e))
            raise e

    def send_obj_to_player(self, obj, player_id):
        self.sio.emit("message", obj, self.connections[player_id])



import pygame

from rtsgame.src.utility.utilities import join_paths, Vector


class Animation:
    def __init__(self, filenames, transforms=None, offset=None):
        if transforms is None:
            transforms = Transforms()
        if offset is not None:
            offset = Vector(*offset)
        self.offset = offset

        images = [pygame.image.load(f) for f in filenames]
        self.images = [transforms(img).convert_alpha() for img in images]

    def __getitem__(self, item: int) -> pygame.Surface:
        return self.images[item]

    def __len__(self):
        return len(self.images)


def parse_descriptions(descriptions):
    animations = {
        row['name']: Animation(join_paths(row['folder'], row['sprites'], ),
                               transforms=get_transforms(row),
                               offset=row['offset'])
        for row in descriptions}

    return animations


class Transform:
    pass


class Transforms(Transform):
    def __init__(self, transforms=None):
        if transforms is None:
            transforms = []

        self.transforms = transforms

    def append(self, transform):
        self.transforms.append(transform)

    def __call__(self, image: pygame.Surface):
        for transform in self.transforms:
            image = transform(image)

        return image


class Flip(Transform):
    def __init__(self, flip_x, flip_y):
        self.flip_x = flip_x
        self.flip_y = flip_y

    def __call__(self, image):
        return pygame.transform.flip(image, self.flip_x, self.flip_y)


class Scale(Transform):
    def __init__(self, factor):
        self.factor = factor

    def __call__(self, img):
        return pygame.transform.scale(img, img.get_height() * self.factor,
                                      img.get_width() * self.factor)


class Resize(Transforms):
    def __init__(self, size):
        super().__init__()
        self.size = size

    def __call__(self, img):
        return pygame.transform.scale(img, self.size)


def get_transforms(config_row):
    transforms = Transforms()
    # Flip transform
    if 'flip_x' in config_row or 'flip_y' in config_row:
        transforms.append(Flip(config_row.get('flip_x', False),
                               config_row.get('flip_y', False)))
    # Resize transform
    if 'resize' in config_row:
        transforms.append(Resize(config_row['resize']))

    return transforms



import logging
import os
from time import time

from .AnimationSystem import AnimationSystem
from .DamageSystem import DamageSystem
from .Entity import Enemy
from .Entity import Projectile
from .GeometrySystem import GeometrySystem
from .SpawnSystem import SpawnSystem
from .WorldState import world


class Logic:
    def __init__(self):
        self.spawn_system = SpawnSystem()
        self.geometry_system = GeometrySystem()
        self.damage_system = DamageSystem()
        self.animation_system = AnimationSystem()
        dirname = os.path.dirname(__file__)
        self.animation_system.load_entity_config(
            os.path.join(dirname, '../utility/animations/'
                                  'melee_animations.json'))
        self.animation_system.load_entity_config(
            os.path.join(dirname, '../utility/animations/'
                                  'player1_animations.json'))
        self.animation_system.load_entity_config(
            os.path.join(dirname, '../utility/animations/'
                                  'player2_animations.json'))

    # move в Logic должен разумно вызывать move GeometrySystem в зависимости от WorldState
    # move в GeometrySystem должен обновлять состояние мира
    # move у Entity - просто делает сдвигает в указанном направлении
    def move(self, entity_id, direction=None):
        if direction is None:
            direction = world.get_direction(entity_id)
        if direction == (0, 0):
            return
        world.set_direction(entity_id, direction)
        dx, dy = [i * world.get_velocity(entity_id) for i in direction]

        # PROJECTILES
        if isinstance(world.entity[entity_id], Projectile):
            temp_box = world.get_box(entity_id).move(dx, dy)
            # Если снаряд попал в стену, то его нужно удалить
            if self.geometry_system.collide_with_wall(temp_box):
                world.dead_entities.append(entity_id)
                return

            # Если наш projectile пересекается с другим entity
            for other_entity_id in world.entity.keys():
                if self.geometry_system.collide(temp_box,
                                                world.get_box(other_entity_id)) \
                        and entity_id != other_entity_id:
                    self.damage_system.deal_damage(entity_id, other_entity_id)

        # NOT-PROJECTILE
        for delta_x, delta_y in [(dx, 0), (0, dy)]:
            temp_box = world.get_box(entity_id).move(delta_x, delta_y)
            if self.geometry_system.collide_with_wall(temp_box):
                continue

            for other_entity_id in world.entity.keys():
                if (self.geometry_system.collide(temp_box, world.get_box(
                        other_entity_id))
                        and entity_id != other_entity_id):
                    break
            else:
                # Если никто ни в кого не врезался - двигаем
                world.set_box(entity_id, temp_box)

    def move_all_entities(self):
        for entity_id in world.movable_entities:
            # Если entity не начал атаку
            if world.get_last_attack(entity_id) is None:
                self.move(entity_id)
                self.animation_system.continue_or_reset_move_animation(
                    entity_id,
                    world.get_direction(entity_id))

    # TODO: different attack types (Melee/Ranged)
    def attack(self, entity_id):
        for other_entity_id in self.geometry_system.get_attackable_entites(
                entity_id):
            self.damage_system.deal_damage(entity_id, other_entity_id)

    def update_attack_state(self):
        for entity_id in world.entity.keys():
            if world.get_last_attack(entity_id) is not None:
                if time() - world.get_last_attack(
                        entity_id) > world.get_attack_reload(entity_id):
                    self.attack(entity_id)
                    world.set_last_attack(entity_id, None)
                    self.animation_system.continue_or_reset_move_animation(
                        entity_id, world.get_direction(entity_id)
                    )

            if isinstance(world.entity[entity_id], Enemy) and len(
                    self.geometry_system.get_attackable_entites(
                        entity_id)) > 0:
                if world.get_last_attack(entity_id) is None:
                    self.start_attack(entity_id,
                                      world.get_direction(entity_id))

    def update_enemies_direcion(self):
        for entity_id in world.enemies:
            if entity_id not in world.dead_entities:
                self.geometry_system.find_aim(entity_id)
                world.set_direction(entity_id,
                                    self.geometry_system.generate_npc_movement(
                                        entity_id))

    def start_attack(self, id_, direction):
        if world.get_last_attack(id_) is not None:
            return
        attack_anim = self.animation_system.get_attack_animation(id_,
                                                                 direction)

        self.animation_system.reset_animation(id_, attack_anim)
        world.set_last_attack(id_, time())
        logging.info("Entity {} attacked".format(id_))



import logging
from random import randint

from pygame import Rect

from .EnemyFactory import MeleeEnemyFactory
from .Entity import PlayerEntity
from .GeometrySystem import GeometrySystem
from .WorldState import world
from ..utility.constants import *


class MonsterSpawner:
    def __init__(self, difficulty):
        self._monster_factories = []
        self.difficulty = difficulty

    def spawn_monster(self, *args):
        size = len(self._monster_factories)
        return self._monster_factories[randint(0, size - 1)].generate_enemy()

    def add_monster_factory(self, factory):
        factory.load_difficulty_from_file(self.difficulty)
        self._monster_factories.append(factory)


class SpawnSystem:
    def __init__(self):
        self._enemy_types = []
        self.monster_spawner = MonsterSpawner("easy")
        self.melee_enemy_factory = MeleeEnemyFactory()
        self.monster_spawner.add_monster_factory(self.melee_enemy_factory)

    def generate_players(self):
        self.create_player(world.get_first_player_id())
        self.create_player(world.get_second_player_id())

    def create_enemy(self):
        entity = self.monster_spawner.spawn_monster()  # Создание
        entity_box = generate_random_free_box(entity.get_box())  # Определение куда его поставить на карте
        entity.set_box(entity_box)

        # Добавление во всевозможные хранилища
        world.entity[entity._id] = entity
        world.movable_entities.add(entity._id)
        world.enemies.add(entity._id)

    def create_player(self, player_id):
        player_box = generate_random_free_box(Rect(0, 0, PLAYER_HEIGHT, PLAYER_WIDTH))
        world.entity[player_id] = PlayerEntity().set_damage(PLAYER_START_DAMAGE).set_velocity(PLAYER_VELOCITY) \
            .set_direction((0, 0)).set_health(PLAYER_HEALTH).set_id(len(world.entity)). \
            set_box(player_box).set_position((player_box.centerx, player_box.centery)). \
            set_attack_reload(PLAYER_RELOAD).set_type("player" + str(player_id + 1))
        world.movable_entities.add(player_id)

    # TODO: система вещей?
    def create_item(self):
        pass


def generate_random_free_box(box):
    while (True):
        x = randint(0, MAP_SCALE * world.map.width - 1)
        y = randint(0, MAP_SCALE * world.map.height - 1)
        new_box = box.move(x, y)
        if not GeometrySystem.collide_with_wall(new_box):
            intersect_flag = False
            logging.debug("Enteties - {}".format(world.entity))
            for id, ent in world.entity.items():
                ent_box = ent.get_box()
                if GeometrySystem.collide(ent_box, new_box):
                    intersect_flag = True
                    break
            if not intersect_flag:
                return new_box



import random

from rtsgame.src.utility.constants import *
from rtsgame.src.utility.utilities import level_to_tile


class Map:
    def room_overlapping(self, room, room_list):
        x = room[0]
        y = room[1]
        w = room[2]
        h = room[3]
        for current_room in room_list:
            if (x < (current_room[0] + current_room[2]) and
                    current_room[0] < (x + w) and
                    y < (current_room[1] + current_room[3]) and
                    current_room[1] < (y + h)):
                return True
        return False

    def gen_room(self):
        x, y, w, h = 0, 0, 0, 0
        w = random.randint(self.min_room_len, self.max_room_len)
        h = random.randint(self.min_room_len, self.max_room_len)
        x = random.randint(1, (self.width - w - 1))
        y = random.randint(1, (self.height - h - 1))
        return [x, y, w, h]

    def join_rooms(self, room_1, room_2):
        sorted_room = [room_1, room_2]
        sorted_room.sort(key=lambda x_y: x_y[0])
        x1 = sorted_room[0][0]
        y1 = sorted_room[0][1]
        w1 = sorted_room[0][2]
        h1 = sorted_room[0][3]
        x1_2 = x1 + w1 - 1
        y1_2 = y1 + h1 - 1
        x2 = sorted_room[1][0]
        y2 = sorted_room[1][1]
        w2 = sorted_room[1][2]
        h2 = sorted_room[1][3]
        x2_2 = x2 + w2 - 1
        y2_2 = y2 + h2 - 1
        if x1 < (x2 + w2) and x2 < (
                x1 + w1):  # Если комнаты накладываются по x
            jx1 = random.randint(x2, x1_2)
            jx2 = jx1
            tmp_y = [y1, y2, y1_2, y2_2]
            tmp_y.sort()
            jy1 = tmp_y[1] + 1
            jy2 = tmp_y[2] - 1
            corridors = self.corridor_between_points(jx1, jy1, jx2, jy2)
            self.corridor_list.append(corridors)
        elif y1 < (y2 + h2) and y2 < (y1 + h1):  # Аналогично по y
            if y2 > y1:
                jy1 = random.randint(y2, y1_2)
                jy2 = jy1
            else:
                jy1 = random.randint(y1, y2_2)
                jy2 = jy1
            tmp_x = [x1, x2, x1_2, x2_2]
            tmp_x.sort()
            jx1 = tmp_x[1] + 1
            jx2 = tmp_x[2] - 1
            corridors = self.corridor_between_points(jx1, jy1, jx2, jy2)
            self.corridor_list.append(corridors)
        else:  # Буква Г
            join = random.randint(0,
                                  1)  # Выбираем сторону в которую повернута буква Г
            if join == 0:  # Неправильная буква Г - вправо-вверх
                if y2 > y1:
                    jx1 = x1_2 + 1
                    jy1 = random.randint(y1, y1_2)
                    jx2 = random.randint(x2, x2_2)
                    jy2 = y2 - 1
                    corridors = self.corridor_between_points(jx1, jy1, jx2,
                                                             jy2, 0)
                    self.corridor_list.append(corridors)
                else:
                    jx1 = random.randint(x1, x1_2)
                    jy1 = y1 - 1
                    jx2 = x2 - 1
                    jy2 = random.randint(y2, y2_2)
                    corridors = self.corridor_between_points(jx1, jy1, jx2,
                                                             jy2, 1)
                    self.corridor_list.append(corridors)
            else:  # Правильная буква Г - вверх вправо
                if y2 > y1:
                    jx1 = random.randint(x1, x1_2)
                    jy1 = y1_2 + 1
                    jx2 = x2 - 1
                    jy2 = random.randint(y2, y2_2)
                    corridors = self.corridor_between_points(jx1, jy1, jx2,
                                                             jy2, 1)
                    self.corridor_list.append(corridors)
                else:
                    jx1 = x1_2 + 1
                    jy1 = random.randint(y1, y1_2)
                    jx2 = random.randint(x2, x2_2)
                    jy2 = y2_2 + 1
                    corridors = self.corridor_between_points(jx1, jy1, jx2,
                                                             jy2, 0)
                    self.corridor_list.append(corridors)

    def __init__(self, width, height, max_rooms, min_room_len, max_room_len,
                 random_connections):
        self.height = height
        self.min_room_len = min_room_len
        self.random_connections = random_connections
        self.max_room_len = max_room_len
        self.width = width
        self.max_rooms = max_rooms
        self.level = []
        for i in range(self.height):
            self.level.append([STONE] * self.width)
        self.room_list = []
        self.corridor_list = []

        for a in range(self.max_rooms * 5):
            tmp_room = self.gen_room()
            if not self.room_list:
                self.room_list.append(tmp_room)
            else:
                tmp_room_list = self.room_list[:]
                if self.room_overlapping(tmp_room, tmp_room_list) is False:
                    self.room_list.append(tmp_room)
            if len(self.room_list) >= self.max_rooms:
                break

        for a in range(len(self.room_list) - 1):
            self.join_rooms(self.room_list[a], self.room_list[a + 1])

        for a in range(self.random_connections):
            room_1 = self.room_list[random.randint(0, len(self.room_list) - 1)]
            room_2 = self.room_list[random.randint(0, len(self.room_list) - 1)]
            self.join_rooms(room_1, room_2)

        for room_num, room in enumerate(self.room_list):  # Комнаты
            for b in range(room[2]):
                for c in range(room[3]):
                    self.level[room[1] + c][room[0] + b] = FLOOR

        for corridor in self.corridor_list:  # Корридоры|
            x1, y1 = corridor[0]
            x2, y2 = corridor[1]
            for width in range(abs(x1 - x2) + 1):
                for height in range(abs(y1 - y2) + 1):
                    self.level[min(y1, y2) + height][
                        min(x1, x2) + width] = FLOOR
            if len(corridor) == 3:
                x3, y3 = corridor[2]
                for width in range(abs(x2 - x3) + 1):
                    for height in range(abs(y2 - y3) + 1):
                        self.level[min(y2, y3) + height][
                            min(x2, x3) + width] = FLOOR

        for row in range(1, self.height - 1):
            for col in range(1, self.width - 1):
                if self.level[row][col] == FLOOR:
                    if self.level[row - 1][col - 1] == STONE:
                        self.level[row - 1][col - 1] = WALL
                    if self.level[row - 1][col] == STONE:
                        self.level[row - 1][col] = WALL

                    if self.level[row - 1][col + 1] == STONE:
                        self.level[row - 1][col + 1] = WALL

                    if self.level[row][col - 1] == STONE:
                        self.level[row][col - 1] = WALL

                    if self.level[row][col + 1] == STONE:
                        self.level[row][col + 1] = WALL

                    if self.level[row + 1][col - 1] == STONE:
                        self.level[row + 1][col - 1] = WALL

                    if self.level[row + 1][col] == STONE:
                        self.level[row + 1][col] = WALL

                    if self.level[row + 1][col + 1] == STONE:
                        self.level[row + 1][col + 1] = WALL
        self._tile = level_to_tile(self.level)

    #  tile_type{0 - free space; 1 - wall}
    def set(self, x, y, tile_type):
        self._tile[x][y] = tile_type

    def get(self, x, y):
        # Саша пров-=ерь
        x = int(x)
        y = int(y)
        return self._tile[x][y]

    def corridor_between_points(self, x1, y1, x2, y2, param=2):
        if x1 == x2 and y1 == y2 or x1 == x2 or y1 == y2:  # Прямой
            return [(x1, y1), (x2, y2)]
        else:
            if param == 1:
                return [(x1, y1), (x1, y2), (x2, y2)]
            elif param == 0:
                return [(x1, y1), (x2, y1), (x2, y2)]


if __name__ == "__main__":
    map = Map(height=60, width=60, max_rooms=7, max_room_len=8, min_room_len=8,
              random_connections=3)
    for i in map._tile:
        for j in i:
            if (j == -1):
                print("#", end="")
            else:
                print(j, end="")
        print()



from .WorldState import world


class DamageSystem:

    def deal_damage(self, attacker_id, receiver_id):
        new_health = world.get_health(receiver_id) - world.get_damage(
            attacker_id)
        if new_health > 0:
            world.set_health(receiver_id, new_health)
        else:
            world.dead_entities.append(receiver_id)



import pygame

from rtsgame.src.Client.UI.Widget import Widget
from rtsgame.src.utility.utilities import Vector


class Window(Widget):
    def __init__(self):
        super().__init__()
        self.sio = None

    def draw(self, surface: pygame.Surface, abs_position: Vector = Vector(0, 0)):
        super().draw(surface, abs_position)

    def accept_events(self, events):
        for event in events:
            self.accept_event(event)

    def accept_event(self, event):
        pass

    def accept_action(self, action):
        pass

    def set_sio(self, sio):
        self.sio = sio



import eventlet

eventlet.monkey_patch()
import logging
import os

from rtsgame.src.Server import Logic
from rtsgame.src.Server.Server import Server
from rtsgame.src.Server.Visitor import Visitor
from rtsgame.src.Server.WorldState import world
import rtsgame.src.Server.Server

player1_connected = False
player2_connected = False


class App:
    def __init__(self, server):
        self.server = server

        self.logic = Logic.Logic()

    def get_all_entity_information(self, entity_id, visitor):
        return world.entity[entity_id].accept(visitor). \
            set_animation_state(
            *self.logic.animation_system.get_animation_state(
                entity_id)).get_action()

    def send_world_state_to_player(self, player_id):
        entities_to_draw = []
        visitor = Visitor()
        spectate_id = player_id
        if world.player_dead[player_id]:
            spectate_id = (player_id + 1) % 2
        entities_to_draw.append(self.get_all_entity_information(spectate_id, visitor))
        for visible_entity_id in self.logic.geometry_system.get_visible_entities(spectate_id):
            if visible_entity_id != spectate_id:
                entities_to_draw.append(
                    self.get_all_entity_information(visible_entity_id, visitor))
        self.server.send_obj_to_player(entities_to_draw, player_id)

    def update(self):

        if rtsgame.src.Server.Server.player_disconnected:
            exit()
        for dead_id in world.dead_entities:
            if dead_id in (world.get_first_player_id(), world.get_second_player_id()):
                self.server.send_obj_to_player(["SPECTATE"], dead_id)
                world.player_dead[dead_id] = True
                if all(world.player_dead):
                    exit()
            self.logic.animation_system.remove_entity(dead_id)
            world.delete_entity(dead_id)
        world.dead_entities = []

        while not self.server.action_queue.empty():
            self.analyze_action(self.server.action_queue.get())
        self.logic.update_enemies_direcion()
        self.logic.move_all_entities()
        self.logic.update_attack_state()

        self.send_world_state_to_player(world.get_first_player_id())
        if world.get_game_mode() == "Multiplayer":
            self.send_world_state_to_player(world.get_second_player_id())

        if len(world.enemies) < 15:
            self.logic.spawn_system.create_enemy()

    def analyze_action(self, action):
        player_id, current_action = action
        if world.player_dead[player_id]:
            return
        if current_action == "PLAYER_CONNECTED":
            self.logic.spawn_system.create_player(player_id)

        if current_action == 'ATTACK':
            self.logic.start_attack(player_id, world.get_direction(player_id))

        if current_action[0] == 'MOVE':
            world.set_direction(player_id, tuple(current_action[1]))
            if world.get_last_attack(player_id) is None:
                self.logic.animation_system.continue_or_reset_move_animation(
                    player_id,
                    world.get_direction(player_id)
                )

    def run(self):
        while True:
            eventlet.sleep(0.1)
            self.update()


def start_game(game_mode="Singleplayer"):
    server = Server()
    server.start()

    if game_mode == "Singleplayer":
        connected_players = [player1_connected]
        world.set_game_mode("Singleplayer")
    if game_mode == "Multiplayer":
        connected_players = [player1_connected, player2_connected]
        world.set_game_mode("Multiplayer")
    while not all(connected_players):
        eventlet.sleep(0.5)
        current_action = server.action_queue.get()
        if current_action[1] == "PLAYER_CONNECTED":
            connected_players[current_action[0]] = True

    app = App(server)
    for id in range(len(connected_players)):
        app.logic.spawn_system.create_player(id)
        app.logic.animation_system.get_animation_state(
            id)  # Create in animation set
    server.send_obj_all_players(["MAP", world.map.level])
    logging.info("Sent map to everyone. waiting")
    while server.action_queue.qsize() < len(connected_players):
        eventlet.sleep(1)

    logging.info("MAP_RECEIVED_ON_SERVER")
    for i in range(len(connected_players)):
        server.action_queue.get()
    for i in range(15):
        app.logic.spawn_system.create_enemy()
    server.send_obj_all_players(["START_GAME"])
    app.run()


if __name__ == '__main__':
    logging.basicConfig(level=logging.DEBUG)
    os.path.dirname(os.path.abspath(__file__))
    start_game()



from pygame import Rect

from src.Server.Entity import PlayerEntity, MeleeEnemy, RangedEnemy, Projectile
from src.Server.Visitor import Visitor
from src.Server.WorldState import world

visitor = Visitor()

player = PlayerEntity().set_box(Rect(0, 0, 20, 20)).set_id(0).set_health(50)

melee_enemy = MeleeEnemy().set_box(Rect(100, 100, 10, 10))
range_enemy = RangedEnemy().set_box(Rect(200, 200, 5, 5))
projectile = Projectile().set_box(Rect(300, 300, 10, 10))
world.entity[0] = player


def test_player_accept():
    assert player.accept(visitor).get_action() == ((0, 0), 'player1', None, None, (50, 100))


def test_melee_enemy_accept():
    assert melee_enemy.accept(visitor).get_action() == ((100, 100), 'melee', None, None, None)


def test_range_enemy_accept():
    assert range_enemy.accept(visitor).get_action() == ((200, 200), 'range', None, None, None)


def test_projectile():
    assert projectile.accept(visitor).get_action() == ((300, 300), 'projectile', None, None, None)



import os

from src.Server.Logic import Logic
from src.Server.WorldState import WorldState, world

os.chdir("..")

def spawn_enemies(count, logic):
    for i in range(count):
        logic.spawn_system.create_enemy()
    assert len(world.entity) == count

def despawn_enemies(count):
    for i in range(count):
        world.delete_entity(i + 2)

def test_world_time_creation(benchmark):
    benchmark(WorldState)

def test_spawn_and_dead_time(benchmark):
    benchmark(spawn_and_delete_enemies, 60)

def spawn_and_delete_enemies(count):
    logic = Logic()
    spawn_enemies(count, logic)
    despawn_enemies(count)

def move_entities(count):
    logic = Logic()
    spawn_enemies(count, logic)
    logic.move_all_entities()
    despawn_enemies(count)

def test_move_entities(benchmark):
    benchmark(move_entities, count=60)




import pygame

from rtsgame.src.Client.UI.Widget import Widget
from rtsgame.src.utility.utilities import Vector


class TextWidget(Widget):
    def __init__(self, text):
        super().__init__()
        font = pygame.font.SysFont('Comic Sans MS', 30)
        self.text = text
        self.text_surface = font.render(text, False, (255, 255, 255))
        self.text_surface = self.text_surface.convert_alpha()

    def draw(self, surface: pygame.Surface, abs_position: Vector):
        print(self.text, abs_position)
        surface.blit(self.text_surface, abs_position)



from setuptools import setup, find_packages


def readme():
    with open('README.md') as f:
        return f.read()


setup(name='rtsgame',
      version='0.6',
      description='The funniest rts in the world',
      long_description=readme(),
      classifiers=[
          'Development Status :: 3 - Alpha',
          'License :: OSI Approved :: MIT License',
          'Programming Language :: Python :: 3',
      ],
      url='https://github.com/ashat1701/rts-game',
      author='Mega team',
      author_email='harmoning.moagic@ya.ru',
      license='MIT',
      packages=find_packages(),
      install_requires=[
          'pygame',
          'eventlet',
          'greenlet',
          'python-engineio',
          'requests',
          'websocket-client',
          'python-socketio'
      ],
      entry_points={
          'console_scripts': ['rtsgamestart=rtsgame.MainMenu:main'],
      },
      include_package_data=True,
      zip_safe=False)



from threading import Lock

import pygame

from rtsgame.src.Client.Camera import Camera
from rtsgame.src.Client.EntitySprite import EntitySprite
from rtsgame.src.Client.TextWidget import TextWidget
from rtsgame.src.Client.UI.Window import Window
from rtsgame.src.utility.constants import MOVE_UPDATE
from rtsgame.src.utility.utilities import Vector


class MainWindow(Window):
    def __init__(self, size, sio=None):
        super().__init__()
        self.sio = sio
        self.entities = []
        self.lock = Lock()
        self.main_camera = Camera(Vector(0, 0), size)
        self.add_child(self.main_camera, Vector(0, 0))
        self.spectate = False
        pygame.time.set_timer(pygame.USEREVENT, MOVE_UPDATE)

    def accept_event(self, event):
        if self.spectate:
            return

        if event.type == pygame.USEREVENT:
            state = pygame.key.get_pressed()
            direction = Vector(0, 0)

            if state[pygame.K_w]:
                direction += Vector(0, -1)
            if state[pygame.K_s]:
                direction += Vector(0, 1)
            if state[pygame.K_d]:
                direction += Vector(1, 0)
            if state[pygame.K_a]:
                direction += Vector(-1, 0)

            self.sio.emit('message', ("MOVE", tuple(direction)))

        if event.type == pygame.KEYDOWN and event.key == pygame.K_f:
            self.sio.emit('message', "ATTACK")

    def enable_spectate(self):
        spectate_text = TextWidget('Spectating')
        self.main_camera.add_child(spectate_text, Vector(0, 900))

        self.spectate = True

    def accept_action(self, action):
        if not isinstance(action, list):
            raise RuntimeError(
                "Action is not of type list. Don't know what to do with it")

        if action[0] == 'SPECTATE':
            self.enable_spectate()
        else:
            self.entities = [EntitySprite(info) for info in action]
            with self.lock:
                self.main_camera.set_hp(self.entities[0].health)
                self.main_camera.set_center(self.entities[0].position)
                self.main_camera.set_sprites(self.entities)



import os

from src.Server.Logic import Logic
from src.Server.WorldState import world

os.chdir("..")
logic = Logic()


def test_delete():
    logic.spawn_system.create_enemy()
    assert len(world.enemies) == 1
    assert len(world.entity) == 1
    assert len(world.movable_entities) == 1
    world.delete_entity(2)
    assert len(world.enemies) == 0
    assert len(world.entity) == 0
    assert len(world.movable_entities) == 0



import json
import logging
import time

from rtsgame.src.Server.WorldState import world


class StaticAnimation:
    def __init__(self, duration: int, frame_num: int, play_once: bool):
        self.duration = duration
        self.frame_num = frame_num
        self.play_once = play_once


class AnimationSet:
    def __init__(self, animations, move_binds, attack_binds, anim):
        self._possible_animations = animations
        self._cur_animation_name = None
        self._cur_frame_start = None
        self._cur_frame = None
        self.reset_animation(anim)
        self._move_binds = move_binds
        self._attack_binds = attack_binds

    def get_state(self):
        time_delta = int(time.time() * 1000) - self._cur_frame_start
        frames_skip = time_delta // self.cur_animation.duration

        if frames_skip > 0:
            self.set_frame(self._cur_frame + frames_skip)

        return self._cur_animation_name, self._cur_frame

    def set_frame(self, new_frame):
        if new_frame >= self.cur_animation.frame_num:
            if self.cur_animation.play_once:
                new_frame = self.cur_animation.frame_num - 1
            else:
                new_frame %= self.cur_animation.frame_num

        self._cur_frame_start = int(time.time() * 1000)
        self._cur_frame = new_frame

    def reset_animation(self, animation_name):
        self._cur_animation_name = animation_name
        self.set_frame(0)

    @property
    def cur_animation(self):
        return self._possible_animations[self._cur_animation_name]

    def get_move_animation(self, direction):
        if self._move_binds is None:
            raise RuntimeError("Direction binds for this animation set were"
                               "not specified in config")
        return self._move_binds[direction]

    def get_attack_animation(self, direction):
        if self._attack_binds is None:
            raise RuntimeError("Direction binds for this animation set were"
                               "not specified in config")
        return self._attack_binds[direction]


class AnimationSetFactory:
    def __init__(self):
        self._set_inits = {}

    def register(self, entity_type, animations, move_binds, attack_binds,
                 default_animation):
        if entity_type in self._set_inits:
            raise RuntimeError("{} Already registered for animation set"
                               "factory".format(entity_type))
        self._set_inits[entity_type] = (animations, move_binds, attack_binds,
                                        default_animation)

    def get_animation_set(self, entity_type):
        animation_set = AnimationSet(*self._set_inits[entity_type])
        return animation_set


class AnimationSystem:
    def __init__(self):
        self._anim_sets = {}
        self.factory = AnimationSetFactory()

    def reset_animation(self, id_, animation_name):
        if id_ not in self._anim_sets:
            self.add_entity(id_)

        self._anim_sets[id_].reset_animation(animation_name)

    def continue_or_reset(self, id_, animation_name):
        if id_ not in self._anim_sets:
            self.add_entity(id_)

        if self._anim_sets[id_].get_state()[0] != animation_name:
            self.reset_animation(id_, animation_name)

    def add_entity(self, id_):
        logging.info("AnimationSystem: Added entity {}".format(id_))
        entity_type = world.entity[id_].get_type()

        self._anim_sets[id_] = self.factory.get_animation_set(entity_type)

    def remove_entity(self, id_):
        del self._anim_sets[id_]

    def get_animation_state(self, id_):
        if id_ not in self._anim_sets:
            self.add_entity(id_)
        return self._anim_sets[id_].get_state()

    def continue_or_reset_move_animation(self, id_, direction):
        required_anim = self.get_move_animation(id_, direction)
        self.continue_or_reset(id_, required_anim)

    def get_move_animation(self, id_, direction):
        logging.debug("id - {} direction - {}".format(id_, direction))
        if id_ not in self._anim_sets:
            self.add_entity(id_)
        return self._anim_sets[id_].get_move_animation(direction)

    def get_attack_animation(self, id_, direction):
        if id_ not in self._anim_sets:
            self.add_entity(id_)
        return self._anim_sets[id_].get_attack_animation(direction)

    def load_entity_config(self, file):
        with open(file) as f:
            config = json.load(f)

        all_animations, move_binds = parse_animation_descriptions(
            config['move'])
        attack_animations, attack_binds = parse_animation_descriptions(
            config['attack'])

        all_animations.update(attack_animations)
        self.factory.register(config['entity_type'],
                              all_animations,
                              move_binds,
                              attack_binds,
                              config['default'])


def parse_animation_descriptions(config):
    animations = _parse_animations(config)
    direction_binds = _parse_direction_binds(config)
    return animations, direction_binds


def _parse_animations(config_obj):
    animations = {}
    for anim in config_obj:
        if anim['name'] in animations:
            raise RuntimeError("Animation {} is specified twice")

        static_anim = StaticAnimation(anim['frame_duration'],
                                      len(anim['sprites']),
                                      anim['play_once'])
        animations[anim['name']] = static_anim

    return animations


def _parse_direction_binds(config_obj):
    binds = {}
    for anim, direction in _iter_all_binds(config_obj):
        if direction in binds:
            raise RuntimeError("Animation to direction {} was binded twice"
                               .format(direction))
        binds[direction] = anim['name']

    if len(binds) != 9 and len(binds) != 0:
        raise RuntimeError("Animation has at least one direction bind in "
                           "which case all of them are required")

    return binds if len(binds) != 0 else None


def _iter_all_binds(config_obj):
    for anim in config_obj:
        for direction in anim.get('direction_binds', []):
            yield anim, tuple(direction)



from rtsgame.src.Server.WorldState import world
from rtsgame.src.utility.constants import PLAYER_HEALTH
from .ActionBuilder import ActionBuilder


class Visitor:
    def visit_player(self, entity):
        return (ActionBuilder().set_position(
            entity.get_box().topleft).set_type("player" + str(entity.get_id() + 1))
                .set_hp(world.get_health(entity.get_id()), PLAYER_HEALTH))

    def visit_melee_enemy(self, entity):
        return ActionBuilder().set_position(entity.get_box().topleft).set_type(
            "melee")

    def visit_ranged_enemy(self, entity):
        return ActionBuilder().set_position(entity.get_box().topleft).set_type(
            "range")

    def visit_projectile(self, entity):
        return ActionBuilder().set_position(entity.get_box().topleft).set_type(
            "projectile")



from ..utility.constants import *


class Entity:
    def __init__(self):
        self._id = None
        self.box = None  # box - это хитбокс (pygame.rect для готовой геометрии)
        self._type = "entity"

    def accept(self, visitor):
        raise NotImplementedError

    def set_id(self, id):
        self._id = id
        return self

    def set_position(self, position):
        self._position = position
        return self

    def get_position(self):
        return self._position

    def get_id(self):
        return self._id

    def set_box(self, box):
        self.box = box
        return self

    def get_box(self):
        return self.box

    def get_type(self):
        return self._type


class MovableEntity(Entity):
    def __init__(self):
        super().__init__()
        self._velocity = None
        self._direction = None

    def set_velocity(self, velocity):
        self._velocity = velocity
        return self

    def set_direction(self, direction):
        self._direction = direction
        return self

    def get_velocity(self):
        return self._velocity

    def get_direction(self):
        return self._direction

    def move(self):
        dx = self._direction[0] * self._velocity
        dy = self._direction[1] * self._velocity
        self.box.move_ip(dx, dy)


class Enemy(MovableEntity):
    def __init__(self):
        super().__init__()
        self._health = None
        self._damage = None
        self._attack_reload = None
        self._last_attack = None
        self._aim_position = None

    def set_health(self, health):
        self._health = health
        return self

    def set_last_attack(self, last_attack):
        self._last_attack = last_attack
        return self

    def get_last_attack(self):
        return self._last_attack

    def set_attack_reload(self, attack_reload):
        self._attack_reload = attack_reload
        return self

    def set_damage(self, damage):
        self._damage = damage
        return self

    def get_attack_reload(self):
        return self._attack_reload

    def get_health(self):
        return self._health

    def get_damage(self):
        return self._damage

    def attack(self):
        raise NotImplementedError

    def get_aim(self):
        return self._aim_position

    def set_aim(self, position):
        self._aim_position = position


class Projectile(MovableEntity):
    def __init__(self):
        super().__init__()
        self._damage = None
        self._health = 0

    def accept(self, visitor):
        return visitor.visit_projectile(self)

    def set_damage(self, damage):
        self._damage = damage
        return self

    def get_damage(self):
        return self._damage

    def get_health(self):
        return self._health


# TODO MAKE LOADING ANIMATIONS MORE ROBUST
class MeleeEnemy(Enemy):
    def __init__(self):
        super().__init__()
        self._type = 'melee'

    def accept(self, visitor):
        return visitor.visit_melee_enemy(self)

    def attack(self):
        pass


class RangedEnemy(Enemy):
    def __init__(self):
        super().__init__()

    def accept(self, visitor):
        return visitor.visit_ranged_enemy(self)

    def attack(self):
        pass


class PlayerEntity(MovableEntity):
    def __init__(self):
        super().__init__()
        self._type = "player"
        self._damage = PLAYER_START_DAMAGE
        self._health = PLAYER_HEALTH
        self._attack_reload = None
        self._last_attack = None

    def set_type(self, type):
        self._type = type
        return self

    def get_type(self):
        return self._type

    def accept(self, visitor):
        return visitor.visit_player(
            self)

    def set_last_attack(self, last_attack):
        self._last_attack = last_attack
        return self

    def get_last_attack(self):
        return self._last_attack

    def set_attack_reload(self, attack_reload):
        self._attack_reload = attack_reload
        return self

    def get_attack_reload(self):
        return self._attack_reload

    def set_damage(self, damage):
        self._damage = damage
        return self

    def get_damage(self):
        return self._damage

    def get_health(self):
        return self._health

    def set_health(self, health):
        self._health = health
        return self



import pygame

from rtsgame.src.Client.UI.Widget import Widget
from rtsgame.src.utility.utilities import Vector, load_sprites, join_paths


class HPBar(Widget):
    sprites = load_sprites(
        join_paths('src/Client/HUD/HP/Value', ['HP_Value_0.png',
                                               'HP_Value_1.png',
                                               'HP_Value_2.png',
                                               'HP_Value_3.png',
                                               'HP_Value_4.png',
                                               'HP_Value_5.png', ]))

    def __init__(self, cur_hp, max_hp):
        super().__init__()
        self.hp = cur_hp
        self.max_hp = max_hp

    def draw(self, surface: pygame.Surface, abs_position: Vector):
        sprite_num = int(self.hp / self.max_hp * (len(self.sprites) - 1))
        surface.blit(self.sprites[sprite_num], abs_position)
